/* Copyright (C) 2005-2006  Emiliano Betti and Marco Cesati
   Copyright (C) 2003-2004  Marco Cesati
   Copyright (C) 2002-2003  Shell Technologies s.r.l.

   Authors:
            Emiliano Betti <emilianobetti@gmail.com>
            Marco Cesati   <marcocesati@gmail.com>

   Driver for the Galil 1800 PCI motion controllers (http://www.galilmc.com/)

   This is a driver for the Linux kernel 2.6. It is based on an older driver
   for the kernel 2.4, which in turn was loosely inspired by Galil's own
   Linux driver, as well as by a bunch of PCI drivers in the kernel source code

   This program is free software; you can redistribute  it and/or modify it
   under  the terms of  the GNU General  Public License as published by the
   Free Software Foundation.
*/


#ifndef _GALIL1800_CORE_H_
#define _GALIL1800_CORE_H_

#ifndef __KERNEL__
#error UserMode programs should include galil1800.h rather than galil1800_core.h
#endif

#include "galil1800.h"

#define GALIL1800_MODULE_NAME        "galil1800"
#define GALIL1800_CLASS_NAME         "galil"
#define PFX                          GALIL1800_MODULE_NAME ": "
#define DEBUG_PFX                    PFX "DEBUG: "

/* print debug managemant */
#   ifdef GALIL1800_DEBUG
#      define GALIL_PRINT_DEBUG(format_string, args...)             \
              printk(KERN_DEBUG DEBUG_PFX format_string, ##args)
#      define GALIL_ONE_PRINT_DEBUG(format_string, args...)         \
              if(printk_ratelimit()){                               \
                 GALIL_PRINT_DEBUG(format_string, ##args);          \
              }
#      define GALIL_PRINT_DEBUG_PID(format_string, args...)         \
              printk(KERN_DEBUG DEBUG_PFX "pid %d: " format_string, \
                                              current->pid, ##args)
#      define GALIL_ONE_PRINT_DEBUG_PID(format_string, args...)     \
              if(printk_ratelimit()) {                              \
                 GALIL_PRINT_DEBUG_PID(format_string, ##args);      \
              }
#   else /* print debug disabled */
#      define GALIL_PRINT_DEBUG(format_string, args...)
#      define GALIL_PRINT_DEBUG_PID(format_string, args...)
#      define GALIL_ONE_PRINT_DEBUG_PID(format_string, args...)
#      define GALIL_ONE_PRINT_DEBUG(format_string, args...)
#   endif
/* for disable single debug print, prefix it with "NO_" ;-) */
#   define NO_GALIL_PRINT_DEBUG(format_string, args...)
#   define NO_GALIL_PRINT_DEBUG_PID(format_string, args...)
#   define NO_GALIL_ONE_PRINT_DEBUG(format_string, args...)
#   define NO_GALIL_ONE_PRINT_DEBUG_PID(format_string, args...)


#include <linux/interrupt.h>
#include <linux/fs.h>
#include <linux/cdev.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/workqueue.h>
#include <linux/slab.h>
#include <linux/device.h>
#include <linux/pci.h>


/*  3 devices file for each card:
    one for command, one for IRQ or other event, one for info in FIFO2 */
enum { GALIL1800_CARD_DEVICE=0, GALIL1800_EVENT_DEVICE,
       GALIL1800_INFO_DEVICE, GALIL1800_MINOR_PER_CARD };


/* proper device file state descriptor */
struct galil1800_devfile_state {
	struct cdev cdev;
	struct class_device class_dev;               /* to register to sysfs */
	mode_t open_mode;          /* the read/write mode of the device file */
	struct semaphore open_sem; /* semaphore for concurrent open requests */
	wait_queue_head_t open_wait;      /* waiting for an open to complete */
};

/* N.B.: class_dev needs to be set to 0,
   but this is made in galil1800_state init */
#define init_GALIL1800_DEVFILE_STATE(df_s) do {  \
	(df_s).open_mode = 0;                    \
	init_MUTEX(&(df_s).open_sem);            \
	init_waitqueue_head(&(df_s).open_wait);  \
}while(0)

/* init the struct galil1800_devfile_state array in struct galil1800_state */
#define init_GALIL1800_DEVFILE_STATE_array(df_s_array_p) do {              \
	int df_s_index = 0;                                                \
	for(; df_s_index < GALIL1800_MINOR_PER_CARD;  df_s_index++)        \
		init_GALIL1800_DEVFILE_STATE((df_s_array_p)[df_s_index]);  \
}while(0)


#include "galil1800_event.h"
#include "galil1800_card.h"

/* main device driver descriptor, one for each registered device */
struct galil1800_state {
	struct list_head devs;    /* links for the list of Galil 1800 devices */
	struct class *sysfs_class; /* point to class for sysfs */
	struct pci_dev *dev;       /* the PCI main device descriptor */
	unsigned long io;          /* base I/O address */
	unsigned long ctrl_reg;    /* control register I/O address */
	unsigned long irq_reg;     /* irq register I/O address */
	unsigned long chan2_reg;   /* secondary channel register I/O address */
/* TODO  unsigned long chan2_dpram; */ /* secondary channel DPRAM I/O address */
	unsigned int irq;          /* IRQ line number */
	spinlock_t hwlock;         /* spin lock for low-level fiddling */
	int index;                 /* index of card devices */
	char serial[GALIL1800_SERIAL_MAXLEN + 1]; /* device serial number */
	struct device_attribute dev_attr_serial; /* export serial by sysfs */

	unsigned int open_devs;   /* sum of how many processes have opened any
				     device file; protected by open_devs_sem */
	struct semaphore open_devs_sem; /* protect open_devs */

	/* device file descriptors */
	struct galil1800_devfile_state devfile_state[GALIL1800_MINOR_PER_CARD];

	/* Specific data for card device */
	struct galil1800_card_wbuffer card_wbuff; /* circular write buffer */
	wait_queue_head_t card_read_wait; /* waiting for data in card device */

	unsigned int write_delay; /* ticks to wait for check available
	                             space in the write fifo */
	unsigned int read_delay; /* ticks to wait for check available
	                            byte in the read fifo */
	unsigned int write_timeout; /* ticks to wait when there is no
	                               space in the write fifo */
	unsigned int read_timeout; /* ticks to wait when there is no
	                              data in the read fifo */
	/* next 4 class_device_attribute export the above 4 fields in sysfs */
	struct class_device_attribute cd_attr_write_delay;
	struct class_device_attribute cd_attr_read_delay;
	struct class_device_attribute cd_attr_write_timeout;
	struct class_device_attribute cd_attr_read_timeout;

	/* Specific data for event device */
	struct galil1800_event_buffer event_buff; /* circular event buffer */
	struct galil1800_irq_stack irq_stack; /* irq stack
				(used between top and bottom half handler) */
	struct tasklet_struct irq_tasklet; /* Excute the bottom half
					      interrupt handler */

	/* Specific data for info device */
	unsigned int fifo2_timeout; /* ticks to wait when there is no data
	                               in the secondary fifo */
	struct class_device_attribute cd_attr_fifo2_timeout; /* export
						fifo2_timeout in sysfs */
};


static
inline ssize_t galil1800_sysfs_serial_show(struct device *dev,
				struct device_attribute *attr, char *buf)
{
	struct galil1800_state *s = dev_get_drvdata(dev);
	return sprintf(buf, "%s\n", s->serial);
}

#endif
